@@ -6,8 +6,7 @@ import logging
 from os import path
 from tempfile import gettempdir
 
-import jwt  # pyjwt
-import pytz
+import jwt  # pip install pyjwt
 from ratelimit import limits, sleep_and_retry
 from requests import Session
 
@@ -20,28 +19,98 @@ API_BASE_URL = "https://api.flumetech.com"
 URL_OAUTH_TOKEN = API_BASE_URL + "/oauth/token"
 API_QUERY_URL = API_BASE_URL + "/users/{user_id}/devices/{device_id}/query"
 API_DEVICES_URL = API_BASE_URL + "/users/{user_id}/devices"
+API_NOTIFICATIONS_URL = API_BASE_URL + "/users/{user_id}/notifications"
 
 LOGGER = logging.getLogger(__name__)
 
 
 def _generate_api_query_payload(scan_interval):
-    utc_now = pytz.utc.localize(datetime.utcnow())
+    datetime_today = datetime.today()
 
-    since_datetime = (utc_now - scan_interval).strftime("%Y-%m-%d %H:%M:00")
-    until_datetime = utc_now.strftime("%Y-%m-%d %H:%M:00")
+    def format_time(time):
+        return time.isoformat(" ", "seconds")
 
-    return {
-        "queries": [
-            {
-                "since_datetime": since_datetime,
-                "until_datetime": until_datetime,
-                "bucket": "MIN",
-                "request_id": "update",
-                "units": "GALLONS",
-                "tz": "UTC",
-            }
-        ]
-    }
+    def format_start_today():
+        return format_time(datetime.combine(datetime_today, datetime.min.time()))
+
+    def format_start_month():
+        return format_time(
+            datetime.combine(datetime_today.replace(day=1), datetime.min.time())
+        )
+
+    def format_start_week():
+        return format_time(
+            datetime.combine(
+                datetime_today - timedelta(days=datetime_today.weekday()),
+                datetime.min.time(),
+            )
+        )
+
+    queries = [
+        {
+            "request_id": "current_interval",
+            "bucket": "MIN",
+            "since_datetime": format_time(datetime_today - scan_interval),
+            "until_datetime": format_time(datetime_today.replace(second=0)),
+            "operation": "SUM",
+            "units": "GALLONS",
+        },
+        {
+            "request_id": "current_min",
+            "bucket": "MIN",
+            "since_datetime": format_time(datetime_today),
+            "operation": "SUM",
+            "units": "GALLONS",
+        },
+        {
+            "request_id": "today",
+            "bucket": "DAY",
+            "since_datetime": format_start_today(),
+            "until_datetime": format_time(datetime_today),
+            "operation": "SUM",
+            "units": "GALLONS",
+        },
+        {
+            "request_id": "week_to_date",
+            "bucket": "DAY",
+            "since_datetime": format_start_week(),
+            "until_datetime": format_time(datetime_today),
+            "operation": "SUM",
+            "units": "GALLONS",
+        },
+        {
+            "request_id": "month_to_date",
+            "bucket": "MON",
+            "since_datetime": format_start_month(),
+            "until_datetime": format_time(datetime_today),
+            "units": "GALLONS",
+        },
+        {
+            "request_id": "last_60_min",
+            "bucket": "MIN",
+            "since_datetime": format_time(datetime_today - timedelta(minutes=60)),
+            "until_datetime": format_time(datetime_today),
+            "operation": "SUM",
+            "units": "GALLONS",
+        },
+        {
+            "request_id": "last_24_hrs",
+            "bucket": "HR",
+            "since_datetime": format_time(datetime_today - timedelta(hours=24)),
+            "until_datetime": format_time(datetime_today),
+            "operation": "SUM",
+            "units": "GALLONS",
+        },
+        {
+            "request_id": "last_30_days",
+            "bucket": "DAY",
+            "since_datetime": format_time(datetime_today - timedelta(days=30)),
+            "until_datetime": format_time(datetime_today),
+            "operation": "SUM",
+            "units": "GALLONS",
+        },
+    ]
+    return {"queries": queries}
 
 
 def _response_error(message, response):
@@ -133,7 +202,6 @@ class FlumeAuth:
 
     def verify_token(self):
         """Check to see if token is expiring in 12 hours."""
-
         token_expiration = datetime.fromtimestamp(self._decoded_token["exp"])
         time_difference = datetime.now() + timedelta(hours=12)
         LOGGER.debug("Token expiration time: %s", token_expiration)
@@ -190,40 +258,25 @@ class FlumeDeviceList:
     """Get Flume Device List from API."""
 
     def __init__(
-        self,
-        username,
-        password,
-        client_id,
-        client_secret,
-        flume_token_file=TOKEN_FILE,
-        http_session: Session = Session(),
-        timeout=DEFAULT_TIMEOUT,
+        self, flume_auth, http_session: Session = Session(), timeout=DEFAULT_TIMEOUT,
     ):
         """Initialize the data object."""
         self._timeout = timeout
         self._http_session = http_session
-        self._flume_auth = FlumeAuth(
-            username,
-            password,
-            client_id,
-            client_secret,
-            flume_token_file,
-            http_session=http_session,
-            timeout=timeout,
-        )
+        self._flume_auth = flume_auth
         self.device_list = self.get_devices()
 
     def get_devices(self):
         """Return all available devices from Flume API."""
 
         url = API_DEVICES_URL.format(user_id=self._flume_auth.user_id)
-        querystring = {"user": "true", "location": "true"}
+        query_string = {"user": "true", "location": "true"}
 
         response = self._http_session.request(
             "GET",
             url,
             headers=self._flume_auth.authorization_header,
-            params=querystring,
+            params=query_string,
             timeout=self._timeout,
         )
 
@@ -232,7 +285,52 @@ class FlumeDeviceList:
         # Check for response errors.
         _response_error("Impossible to retreive devices", response)
 
-        return json.loads(response.text)["data"]
+        return response.json()["data"]
+
+
+class FlumeNotificationList:
+    """Get Flume Notifications list from API."""
+
+    def __init__(
+        self,
+        flume_auth,
+        http_session: Session = Session(),
+        timeout=DEFAULT_TIMEOUT,
+        read="false",
+    ):
+        """Initialize the data object."""
+        self._timeout = timeout
+        self._http_session = http_session
+        self._flume_auth = flume_auth
+        self._read = read
+        self.notification_list = self.get_notifications()
+
+    def get_notifications(self):
+        """Return all notifications from devices owned by the user"""
+
+        url = API_NOTIFICATIONS_URL.format(user_id=self._flume_auth.user_id)
+
+        query_string = {
+            "limit": "50",
+            "offset": "0",
+            "sort_direction": "ASC",
+            "read": self._read,
+        }
+
+        response = self._http_session.request(
+            "GET",
+            url,
+            headers=self._flume_auth.authorization_header,
+            params=query_string,
+            timeout=self._timeout,
+        )
+
+        LOGGER.debug("get_notifications Response: %s", response.text)
+
+        # Check for response errors.
+        _response_error("Impossible to retrieve notifications", response)
+        print(json.dumps(response.json()))
+        return response.json()["data"]
 
 
 class FlumeData:
@@ -240,34 +338,24 @@ class FlumeData:
 
     def __init__(
         self,
-        username,
-        password,
-        client_id,
-        client_secret,
+        flume_auth,
         device_id,
-        time_zone,
         scan_interval,
-        flume_token_file=TOKEN_FILE,
         update_on_init=True,
         http_session: Session = Session(),
         timeout=DEFAULT_TIMEOUT,
+        query_payload=None,
     ):
         """Initialize the data object."""
         self._http_session = http_session
         self._timeout = timeout
-        self._flume_auth = FlumeAuth(
-            username,
-            password,
-            client_id,
-            client_secret,
-            flume_token_file,
-            http_session=http_session,
-            timeout=timeout,
-        )
-
+        self._flume_auth = flume_auth
         self._scan_interval = scan_interval
         self.device_id = device_id
-        self.value = None
+        self.values = {}
+        if query_payload is None:
+            self._query_payload = _generate_api_query_payload(self._scan_interval)
+        self._query_keys = [q["request_id"] for q in self._query_payload["queries"]]
         if update_on_init:
             self.update()
 
@@ -275,21 +363,24 @@ class FlumeData:
     @limits(calls=2, period=API_LIMIT)
     def update(self):
         """Return updated value for session."""
+        return self.update_force()
+
+    def update_force(self):
+        """Return updated value for session without auto retry or limits."""
         self._flume_auth.read_token_file()
 
-        json_payload = _generate_api_query_payload(self._scan_interval)
         url = API_QUERY_URL.format(
             user_id=self._flume_auth.user_id, device_id=self.device_id
         )
         response = self._http_session.post(
             url,
-            json=json_payload,
+            json=self._query_payload,
             headers=self._flume_auth.authorization_header,
             timeout=self._timeout,
         )
 
         LOGGER.debug("Update URL: %s", url)
-        LOGGER.debug("Update json_payload: %s", json_payload)
+        LOGGER.debug("Update query_payload: %s", self._query_payload)
         LOGGER.debug("Update Response: %s", response.text)
 
         # Check for response errors.
@@ -297,4 +388,6 @@ class FlumeData:
             f"Can't update flume data for user id {self._flume_auth.user_id}", response
         )
 
-        self.value = json.loads(response.text)["data"][0]["update"][0]["value"]
+        responses = response.json()["data"][0]
+
+        self.values = {k: responses[k][0]["value"] for k in self._query_keys}
